README.rst

# ka_uts_log

## Overview

Log Management

## Installation

The package `ka_uts_log` can be installed from PyPI or Anaconda.

To install with `pip`:

    $ python -m pip install ka_uts_log

To install with `conda`:

    $ conda install -c conda-forge ka_uts_log

## Package logging

c.f.: Appendix: Package Logging

## Package files

### Classification

The Package `ka_uts_log` consist of the following file types (c.f.: Appendix):

1. Special files: (c.f.: Appendix: Special python package files)
2. Dunder modules: (c.f.: Appendix: Special python package modules)
3. Modules
   1. log.py
4. Special Sub-directories: (c.f.: Appendix: Special python package Sub-directories)
   1. cfg
      1. __init__.py
      2. log.std.yml
      3. log.usr.yml

### Special Sub-directory: cfg

#### Overview

> Special Sub directory cfg: Files
> Name | Description
> --- | ---
> log.std.yml | Yaml definition file for standard logging
> log.usr.yml | Yaml definition file for user logging

### Modules

The Package `ka_uts_log` contains the following modules.

> ka_uts_log Modules
> Name | Decription
> --- | ---
> log.py | Logging management module

### Module: log.py of Package: ka_uts_log

The Module `log` contains the following static classes.

#### Classes of Module: log

> Classes of Modul log
> Name | Description
> --- | ---
> LogEq | Management of Log Equate message, generated from a key-, value-pair.
> LogDic | Management of Log Equate messages, generated from the key-, value-pairs of a dictionary.
> Log | Management of Log messages

#### Class: Log of Modul: log.py

The static Class `Log` contains the subsequent display- and management-methods.

##### Display Methods of Class: Log

> Log Display Methods
> Name | Description
> --- | ---
> debug | Log debug message to debug destination.
> info | Log info message to info destination.
> warning | Log warnning message to warning destination.
> error | Log error message to error destination.
> critcial | Log critical message to critical destination.

##### Management Methods of Class: Log

> Log Management Methods
> Name | Description
> --- | ---
> init | initialise current class.
> sh_calendar_ts | Show timestamp or datetime.
> sh_dir_run | Show run directory.
> sh_d_dir_run | Show dictionary of run directories.
> sh_d_log_cfg | Show log configuration directory.
> sh_path_log_cfg | Show path of log configuration file.
> sh | initialise and show current class.

##### Management Method: init of Class: Log

###### Parameter

> Log method init: Parameter
> Name | Type | Description
> --- | --- | ---
> cls | class | current class
> **kwargs | TyAny | keyword arguments

#### Class: LogEq of Modul: Log

The static Class `LogEq` of Module log.py contains the subsequent methods

##### Methods of Class LogEq

> LogEq Methods
> Name | Description
> --- | ---
> debug | Log generated equate message "<key> = <value>" to the debug destination.
> info | Log generated equate message "<key> = <value>" to the info destination.
> warning | Log generated equate message "<key> = <value>" to the warning destination.
> error | Log generated equate message "<key> = <value>" to the error destination.
> critcial | Log generated equate message "<key> = <value>" to the critical destination.

All Methods of Class: LogEq use the following Parameters:

###### Methods Parameter of Class: LogEq

> LogEq Methods parameter
> Name | Type | Description
> --- | --- | ---
> cls | class | current class
> key | TyAny | Key
> value | TyAny | Value

#### Class: LogDic of Modul: log.py

The static Class `LogDic` of Module log.py contains the subsequent methods

##### Methods of Class: LogDic

> LogDic Methods
> Name | Description
> --- | ---
> debug | Log generated equate messages for all dictionary entries to the debug destination.
> info | Log generated equate messages for all dictionary entries to the info destination.
> warning | Log generated equate messages for all dictionary entries to the warning destination.
> error | Log generated equate messages for all dictionary entries to the error destination.
> critical | Log generated equate messages for all dictionary entries to the critical destination.

All Methods of Class LogDic use the following Parameters:

###### Method Parameters of Class: LogDic

> LogDic Methods Parameter
> Name | Type | Description
> --- | --- | ---
> cls | class | current class
> dic | TyDic | Dictionary

# Appendix

## Package Logging

### Description

The Standard or user specifig logging is carried out by the log.py module of the logging package ka_uts_log using the standard- or user-configuration files in the logging package configuration directory:

- <logging package directory>/cfg/ka_std_log.yml,
- <logging package directory>/cfg/ka_usr_log.yml.

The Logging configuration of the logging package could be overriden by yaml files with the same names in the application package- or application data-configuration directories:

- <application package directory>/cfg
- <application data directory>/cfg.

### Log message types

Logging defines log file path names for the following log message types: .

1. debug
2. info
3. warning
4. error
5. critical

#### Log types and Log directories

Single or multiple Application log directories can be used for each message type:

> Log types and directoriesg
> Log type | Log directory
> --- | ---
> long | short | multiple | single
> debug | dbqs | dbqs | logs
> info | infs | infs | logs
> warning | wrns | wrns | logs
> error | errs | errs | logs
> critical | crts | crts | logs

#### Application parameter for logging

> Application parameter used in log naming
> Name | Decription | Values | Example
> --- | --- | --- | ---
> dir_dat | Application data directory |  | /otev/data
> tenant | Application tenant name |  | UMH
> package | Application package name |  | otev_xls_srr
> cmd | Application command |  | evupreg
> pid | Process ID |  | 681025
> log_ts_type | Timestamp type used in logging files|ts, dt | ts, dt' | ts
> log_sw_single_dir | Enable single log directory or multiple log directories | True, False | True

#### Log files naming

##### Naming Conventions

> Naming conventions for logging file paths
> Type | Directory | File
> --- | --- | ---
> debug | /<dir_dat>/<tenant>/RUN/<package>/<cmd>/<Log directory> | <Log type>_<ts>_<pid>.log
> info | /<dir_dat>/<tenant>/RUN/<package>/<cmd>/<Log directory> | <Log type>_<ts>_<pid>.log
> warning | /<dir_dat>/<tenant>/RUN/<package>/<cmd>/<Log directory> | <Log type>_<ts>_<pid>.log
> error | /<dir_dat>/<tenant>/RUN/<package>/<cmd>/<Log directory> | <Log type>_<ts>_<pid>.log
> critical | /<dir_dat>/<tenant>/RUN/<package>/<cmd>/<Log directory> | <Log type>_<ts>_<pid>.log

##### Naming Examples

> Naming examples for logging file paths
> Type | Directory | File
> --- | --- | ---
> debug | /data/otev/umh/RUN/otev_xls_srr/evupreg/logs | debs_1737118199_9470.log
> info | /data/otev/umh/RUN/otev_xls_srr/evupreg/logs | infs_1737118199_9470.log
> warning | /data/otev/umh/RUN/otev_xls_srr/evupreg/logs | wrns_1737118199_9470.log
> error | /data/otev/umh/RUN/otev_xls_srr/evupreg/logs | errs_1737118199_9470.log
> critical | /data/otev/umh/RUN/otev_xls_srr/evupreg/logs | crts_1737118199_9470.log

## Python Terminology

### Python Packages

#### Overview

> Python Packages Overview
> Name | Definition
> --- | ---
> Python package | Python packages are directories that contains the special module `__init__.py` and other modules, packages files or directories.
> Python sub-package | Python sub-packages are python packages which are contained in another pyhon package.
> Python package sub-directory | directory contained in a python package.
> Python package special sub-directory | Python package sub-directories with a special meaning like data or cfg

#### Examples

> Python Package sub-directory-Examples
> Name | Description
> --- | ---
> bin | Directory for package scripts.
> cfg | Directory for package configuration files.
> data | Directory for package data files.
> service | Directory for systemd service scripts.

### Python package files

#### Overview

> Python package overview files
> Name | Definition
> --- | ---
> Python package files | Files within a python package.
> Special python package files | Package files which are not modules and used as python and used as python marker files like `__init__.py`.
> Python package module | Files with suffix `.py`; they could be empty or contain python code; other modules can be imported into a module.
> Special python package module | Modules like `__init__.py` or `main.py` with special names and functionality.

#### Examples

> Python package examples files
> Name | Type | Description
> --- | --- | ---
> py.typed | Type checking marker file | The `py.typed` file is a marker file used in Python packages to indicate that the package supports type checking. This is a part of the PEP 561 standard, which provides a standardized way to package and distribute type information in Python.
> __init__.py | Package directory marker file | The dunder (double underscore) module `__init__.py` is used to execute initialisation code or mark the directory it contains as a package. The Module enforces explicit imports and thus clear namespace use and call them with the dot notation.
> __main__.py | entry point for the package | The dunder module `__main__.py` serves as an entry point for the package. The module is executed when the package is called by the interpreter with the command python -m <package name>.
> __version__.py | Version file | The dunder module `__version__.py` consist of assignment statements used in Versioning.

### Python methods

#### Overview

> Python methods overview
> Name | Description
> --- | ---
> Python method | Python functions defined in python modules.
> Special python method | Python functions with special names and functionalities.
> Python class | Classes defined in python modules.
> Python class method | Python methods defined in python classes

#### Examples

> Python methods examples
> Name | Type | Description
> --- | --- | ---
> __init__ | class object constructor method | The special method `__init__` is called when an instance (object) of a class is created; instance attributes can be defined and initalized in the method.

# Table of Contents

Table of Content

- ka_uts_log
  - Overview
  - Installation
  - Package logging
  - Package files
    - Classification
    - Special Sub-directory: cfg
      - Overview
    - Modules
    - Module: log.py of Package: ka_uts_log
      - Classes of Module: log
      - Class: Log of Modul: log.py
        - Display Methods of Class: Log
        - Management Methods of Class: Log
        - Management Method: init of Class: Log
          - Parameter
      - Class: LogEq of Modul: Log
        - Methods of Class LogEq
          - Methods Parameter of Class: LogEq
      - Class: LogDic of Modul: log.py
        - Methods of Class: LogDic
          - Method Parameters of Class: LogDic
- Appendix
  - Package Logging
    - Description
    - Log message types
      - Log types and Log directories
      - Application parameter for logging
      - Log files naming
        - Naming Conventions
        - Naming Examples
  - Python Terminology
    - Python Packages
      - Overview
      - Examples
    - Python package files
      - Overview
      - Examples
    - Python methods
      - Overview
      - Examples
- Table of Contents